--------------------------------------------------------------------------------
-- Tool: Bubble Machine
-- by Kiffer-Opa
-- version 0.5
--------------------------------------------------------------------------------

-- Setup Tables
if kifferopa==nil then kifferopa={} end
kifferopa.bubblegen={}
kifferopa.bubblegen.gfx_wpn={}
kifferopa.bubblegen.gen={}
kifferopa.bubblegen.bubble={}
kifferopa.bubblegen.tempparent=nil

-- Load & Prepare Ressources
kifferopa.bubblegen.sfx_build=loadsfx("buildmetal.ogg")						-- Generator Build Sound
kifferopa.bubblegen.sfx_bounce=loadsfx("bounce.wav")						-- Bounce
kifferopa.bubblegen.sfx_bubblebounce=loadsfx("bubble_bounce.wav")
kifferopa.bubblegen.sfx_bubbleburst=loadsfx("bubble_burst.ogg")
kifferopa.bubblegen.sfx_bubbleinflation=loadsfx("bubble_inflation.ogg")
kifferopa.bubblegen.gfx_icon=loadgfx("buildings/bubblemachine.png")
setmidhandle(kifferopa.bubblegen.gfx_icon)
kifferopa.bubblegen.gfx_generator=loadgfx("buildings/bubblemachine.png")
setmidhandle(kifferopa.bubblegen.gfx_generator)
kifferopa.bubblegen.gfx_bubble=loadgfx("buildings/soapbubble128.png")
setmidhandle(kifferopa.bubblegen.gfx_bubble)
kifferopa.bubblegen.gfx_bubblemask=loadgfx("buildings/soapbubble128_mask.png")
setmidhandle(kifferopa.bubblegen.gfx_bubblemask)

--------------------------------------------------------------------------------
-- Weapon: Shield Generator
--------------------------------------------------------------------------------

kifferopa.bubblegen.id=addweapon("kifferopa.bubblegen","Bubble Machine",kifferopa.bubblegen.gfx_icon,0) -- Add Weapon (0 uses)
kifferopa.bubblegen.shields=1										-- Shield Amount

function kifferopa.bubblegen.draw()									-- Draw
	if weapon_shots<1 then
		-- HUD Positioning
		if weapon_shots==0 then
			-- Generator
			hudinfo("Place the generator!")
			hudpositioning(pos_build,kifferopa.bubblegen.gfx_generator)
		end
	end
end

function kifferopa.bubblegen.attack(attack)												-- Attack
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots<1) and (weapon_position==1) then
		weapon_position=0
		-- Use weapon
		if weapon_shots==0 then useweapon(0) end
		weapon_shots=weapon_shots+1
		-- Build
		if weapon_shots==1 then
			-- Build Generator
			createobject(kifferopa.bubblegen.gen.id,weapon_x,weapon_y)
			-- Sound
			playsound(kifferopa.bubblegen.sfx_build)
		end
		-- End Turn
		if weapon_shots>=1 then endturn() end
	end
end

--------------------------------------------------------------------------------
-- Object: Generator
--------------------------------------------------------------------------------

kifferopa.bubblegen.gen.id=addobject("kifferopa.bubblegen.gen",kifferopa.bubblegen.gfx_generator)			-- Add Object
function kifferopa.bubblegen.gen.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=50
	objects[id].sy=0
	objects[id].child=nil
	-- Save ID in tempparent variable
	kifferopa.bubblegen.tempparent=id
end
function kifferopa.bubblegen.gen.draw(id,x,y)
	-- Smoke Particles?
	if objects[id].health<=30 then
		if getframe()%3==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
			particlecolor(25,25,25)
			if getframe()%12==0 then
				particle(p_spark,x-16,y-2+math.random(-4,5))
			end
		end
	else
		local child=objects[id].child
		if(child~=nil) then
			if(objects[child].scale>=1) then
				if getframe()%1==0 then
					for i=1,9 do
						particle(p_fragment,x+math.random(-7,6),y-11)
						particlespeed(math.random(-5,5),math.random(-5,5))
						particlefadealpha(0.005)
						particlealpha(0.05)
						particlecolor(objects[child].r,objects[child].g,objects[child].b)
					end
				end
			end
		end		
	end
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(0)
	-- Draw
	drawimage(kifferopa.bubblegen.gfx_generator,x,y)
end
function kifferopa.bubblegen.gen.update(id,x,y)
	local collided=false
	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(kifferopa.bubblegen.gfx_generator,x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(kifferopa.bubblegen.sfx_bounce) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
			collided=true
		end
		-- Water
		if y>getwatery()-5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove Generator
			removeobject(id)
			-- Remove the bubble 
			kifferopa.bubblegen.gen.removebubble(objects[id].child)
			break
		end
	end

	-- create the bubble if the machine stands on the ground for the first time
	if(math.abs(objects[id].sy)<0.05 and collided==true) then
		if(kifferopa.bubblegen.tempparent~=nil and objects[id].child==nil) then		
			createobject(kifferopa.bubblegen.bubble.id,weapon_x,weapon_y,kifferopa.bubblegen.tempparent)
			kifferopa.bubblegen.tempparent=nil
		end
	end


	objectposition(id,x,y)
	
	-- block turn end if child does not exist
	if(objects[id].child==nil) then
		return 1
	end
end
function kifferopa.bubblegen.gen.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy terrain
			terrainexplosion(x,y,30,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,x,y,math.random(6,9)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,x,y,math.random(6,9)*0.1,grey,grey,grey)
			end
			-- Remove Generator
			removeobject(id)
			-- Remove the bubble
			kifferopa.bubblegen.gen.removebubble(objects[id].child)
			-- Cause damage
			arealdamage(x,y,100,50)
		end
	end
end
function kifferopa.bubblegen.gen.removebubble(id)
	particle(p_ring,getobjectx(id),getobjecty(id))
	particlesize(1,1)
	particlecolor(objects[id].r,objects[id].g,objects[id].b)
	particlealpha(0.5)
	particlefadealpha(0.1)
	playsound(kifferopa.bubblegen.sfx_bubbleburst)
	--Remove
	removeobject(id)
end

--------------------------------------------------------------------------------
-- Object: Bubble 
--------------------------------------------------------------------------------
kifferopa.bubblegen.bubble.id=addobject("kifferopa.bubblegen.bubble",0)


function kifferopa.bubblegen.bubble.draw(id,x,y)
	objects[id].r=155+math.floor(math.abs(math.sin(os.clock()*5)*0.3)*100)
	objects[id].g=155+math.floor(math.abs(math.cos(os.clock()*6)*0.3)*100)
	objects[id].b=170+math.floor(math.abs(math.sin(os.clock()*4)*0.3)*85)
	setcolor(objects[id].r,objects[id].g,objects[id].b)
	setblend(blend_alpha)
	setalpha(0.7-math.abs(math.sin(id*10+os.clock()*5)*0.1))
	setscale(objects[id].scale+math.abs(math.sin(math.rad(objects[id].bounceanim)))*0.1,objects[id].scale+math.cos(math.sin(math.rad(objects[id].bounceanim)))*0.1)
	setrotation(0)
	drawimage(kifferopa.bubblegen.gfx_bubble,x,y)	
end

function kifferopa.bubblegen.bubble.setup(id,parameter)
	objects[id]={}
	objects[id].parent=parameter
	objects[id].bounceanim=90
	objects[id].r=math.random(155,255)
	objects[id].g=math.random(155,255)
	objects[id].b=math.random(170,255)
	objects[id].scale=0.0
	objects[parameter].child=id
	playsound(kifferopa.bubblegen.sfx_bubbleinflation)
end

function kifferopa.bubblegen.bubble.update(id,x,y)
	local generator=objects[id].parent
	local parentx=getobjectx(generator)
	local parenty=getobjecty(generator)
	objectposition(id,parentx,parenty)

	if(objects[id].scale<1) then
		objects[id].scale=objects[id].scale+0.012
	elseif(objects[id].scale>1) then
		objects[id].scale=1
	end

	if(objects[id].bounceanim>90) then
		objects[id].bounceanim=objects[id].bounceanim-10
	end

	-- collisions
	if(objects[id].scale>=1) then
		if collision(kifferopa.bubblegen.gfx_bubblemask,x,y,0,1,1,-1,id)==1 then
			local o=objectcollision()
			if(o>0 and getobjecttype(o)~=kifferopa.bubblegen.gen.id) then
				objectposition(o,getobjectx(o)-getobjectxspeed(o),getobjecty(o)-getobjectyspeed(o))
			end
			local p=playercollision()
			if(p~=0) then
				if(getplayerxspeed(p)>1.0) or (getplayeryspeed(p)>1.0) then
					if(objects[id].bounceanim<100) then
						playsound(kifferopa.bubblegen.sfx_bubblebounce)
						objects[id].bounceanim=270
					end
				end
			end
		end
		

		if projectiles~=nil then
			for k,v in ipairs(projectiles) do
				if((v.vx~=nil and v.vy~=nil) or (v.sx~=nil and v.sy~=nil)) and (v.x~=nil and v.y~=nil) then
					local tmpx, tmpy
					local tmpsx, tmpsx
					local dx,dy
					if(v.sx~=nil) then
						tmpsx=v.sx
						tmpsy=v.sy
					end
					if(v.vx~=nil) then
						tmpsx=v.vx
						tmpsy=v.vx
					end
					tmpx=v.x
					tmpy=v.y
					pro_rot=math.atan2(tmpsy,tmpsx)
					dx=tmpx-x
					dy=tmpy-y
					coord_rot=math.atan2(dy,dx)
					msubt=math.ceil(math.max(math.abs(tmpsx,math.abs(tmpsy))/3))
					msubx=tmpsx/msubt
					msuby=tmpsy/msubt
					tmpI=0
					if(math.abs(pro_rot-coord_rot)>math.pi/2) then
						for i=1,msubt,1 do
							if((tmpx+msubx)>=(x-64) and (tmpy+msuby)>=(y-64) and (tmpx+msubx)<=(x+64) and (tmpy+msuby)<=(y+64)) then
								if( ((tmpx+msubx)<=(x-0) or (tmpx+msubx)>=(x+0)) and ((tmpy+msuby)<=(y-0) or (tmpy+msuby)>=(y+0)) ) then
									tmpI=tmpI+1
									if tmpI>1000 then break end
									if(v.sx~=nil) then
										v.sx=-v.sx*0.5
										v.sy=-v.sy*0.5
									end
									if(v.vx~=nil) then
										v.vx=-v.vx*0.5
										v.vy=-v.vy*0.5
									end
									if(objects[id].bounceanim<100) then
										playsound(kifferopa.bubblegen.sfx_bubblebounce)
										objects[id].bounceanim=270
									end
									break
								end	
							end
							tmpx=tmpx+msubx
							tmpy=tmpy+msuby
					
						end
					end
				end
			end	
		end
	else	-- if bubble has not full size
		-- then block turn end (until it has full size)
		return 1
	end	
end


function kifferopa.bubblegen.bubble.damage(id,x,y)
	return nil
end
